/* Katoob
 * Copyright (c) 2002,2003 Arabeyes, Mohammed Sameer.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "katoob.h"
#include <langinfo.h>		/* nl_langinfo() */
#include <gtk/gtk.h>
#include <glib.h>
#include <string.h>		/* strcmp() */
#include "encodings.h"
#include "encodingslist.h"
#include "katoobdocument.h"
#include "mdi.h"
#include "misc.h"
#include "file.h"

gint coverted_ok = -1;		/* -1: unk, 0 Ok, 1: Error */

static void katoob_check_default_encoding ();
static gint katoob_encodings_get_number ();
static GtkWidget *katoob_generate_encodings_menu ();
static GList *katoob_generate_encodings_list ();

gboolean katoob_switch_encoding (GtkCheckMenuItem * checkmenuitem,
				 gpointer user_data);

static gint
katoob_encodings_get_number ()
{
  gint no = 0;
  Lang *tmp = languages;
  while (tmp->name)
    {
      Lang *tmp2 = tmp->children;
      while (tmp2->name)
	{
	  ++no;
	  ++tmp2;
	}
      ++tmp;
    }
  return no;
}

void
encodings_init ()
{
  extern conf *config;
  extern UI *katoob;
/* NOTE: I can't generate the list without generating the menu first */
  katoob->encodings_menu = katoob_generate_encodings_menu ();
  config->enc = katoob_generate_encodings_list ();
  katoob_check_default_encoding ();
}

static void
katoob_check_default_encoding ()
{
/******************************************************************************
 * If the user want to get the encodding from the current locale, Or a        *
 * specialised encoding, That's OK. BUT we have to check if it's supported or *
 * not and use it. Or use our default encoding.                               *
 ******************************************************************************/
  extern conf *config;
  if (config->locale_enc)
    {
      katoob_debug (nl_langinfo (CODESET));
      config->defenc = katoob_encodings_get_by_name (nl_langinfo (CODESET));
    }
  else if (config->special_enc)
    {
      config->defenc = katoob_encodings_get_by_name (config->saved_enc);
    }

/* Validate that that's a supported encoding and fall back to our default */

  if ((config->defenc > katoob_encodings_get_number ())
      || (config->defenc < 0))
    {
      config->defenc = katoob_encodings_get_by_name ("WINDOWS-1256");
    }
}

gchar *
katoob_encodings_get_by_number (gint no)
{
  if ((no >= 0) && (no < katoob_encodings_get_number ()))
    {
      return encodings[no];
    }
  else
    {
      return NULL;
    }
}

gchar *
katoob_encodings_get_name_by_number (gint no)
{
  if ((no >= 0) && (no < katoob_encodings_get_number ()))
    {
      return encodings_names[no];
    }
  else
    {
      return NULL;
    }
}

gint
katoob_encodings_get_by_name (gchar * name)
{
  gint x = 0;
  for (x = 0; x < katoob_encodings_get_number (); x++)
    {
      if (!strcmp (name, encodings[x]))
	{
	  return x;
	}
    }
  return -1;
}

gint
katoob_encodings_get_name_from_string (gchar * name)
{
  gint x = 0;
  for (x = 0; x < katoob_encodings_get_number (); x++)
    {
      if (!strcmp (name, encodings_names[x]))
	{
	  return x;
	}
    }
  return -1;
}

static GtkWidget *
katoob_generate_encodings_menu ()
{
  extern UI *katoob;

  GtkWidget *menu;
  GtkWidget *item;
  GtkWidget *sub;
  GtkWidget *sub_menu;
  GtkWidget *tearoff;
  GSList *group = NULL;
  gint i = 0;
  Lang *tmp = languages;

  menu = gtk_menu_new ();
  tearoff = gtk_tearoff_menu_item_new ();
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), tearoff);

  while (tmp->name)
    {
      Lang *tmp2 = tmp->children;
      sub = gtk_menu_item_new_with_label (tmp->name);
      sub_menu = gtk_menu_new ();
      gtk_menu_item_set_submenu (GTK_MENU_ITEM (sub), sub_menu);
      gtk_menu_shell_append (GTK_MENU_SHELL (menu), sub);

      while (tmp2->name)
	{
	  encodings[i] = tmp2->encoding;
	  encodings_names[i] = tmp2->name;
	  katoob->encodings[i] = item =
	    gtk_radio_menu_item_new_with_label (group, tmp2->name);
	  group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
	  gtk_container_add (GTK_CONTAINER (sub_menu), item);
	  g_signal_connect_after (G_OBJECT (katoob->encodings[i]), "toggled",
				  G_CALLBACK (katoob_switch_encoding),
				  (gpointer) i);
	  ++i;
	  ++tmp2;
	}
      ++tmp;
    }
  return menu;
}

static GList *
katoob_generate_encodings_list ()
{
  gint x = 0;
  GList *enc = NULL;
  gint n = katoob_encodings_get_number ();

  for (x = 0; x < n; x++)
    {
      enc = g_list_append (enc, katoob_encodings_get_name_by_number (x));
    }
  return enc;
}

gboolean
katoob_switch_encoding (GtkCheckMenuItem * checkmenuitem, gpointer user_data)
{
  extern UI *katoob;
  KatoobDocument *doc;
  gchar *_tmp;

  katoob_debug (__FUNCTION__);

  if (!katoob->win)
    {
      return TRUE;		/* Stop processing. */
    }

  doc = katoob_get_active_doc ();

  katoob_debug (__FUNCTION__);

  if (!doc)
    {
      return TRUE;
    }

/* Don't update if it's the same encoding. */
  if ((!GTK_CHECK_MENU_ITEM (checkmenuitem)->active) ||
      (GPOINTER_TO_INT (user_data) == katoob_document_get_encoding (doc)))
    {
      return TRUE;
    }

  _tmp = g_strdup_printf ("Target encoding: %i", GPOINTER_TO_INT (user_data));
  katoob_debug (_tmp);
  g_free (_tmp);
/****************************************************************************
 * Change the document encoding, We should have a signal connected to catch *
 * this event, do the actual work and update the statusbar                  *
 ****************************************************************************/
  katoob_document_set_encoding (doc, GPOINTER_TO_INT (user_data));

  katoob_debug ("katoob_switch_encoding(): still here!");

  if (coverted_ok == 0)
    {
      coverted_ok = -1;
      return FALSE;
    }
  else if (coverted_ok == 1)
    {
      coverted_ok = -1;
      return TRUE;
    }
  else
    {
      g_warning ("coverted_ok Should never be -1 here\n");
      return TRUE;
    }
}

void
katoob_encoding_changed_cb (KatoobDocument * doc, gint enc)
{
/*******************************************************************************
 * If the document is not associated with a file, Convert the text to the      *
 * Then convert it back to the choosen encoding.                               *
 * If there is a file, and the document is modified, we prompt the user        *
 *      asking what to do.                                                     *
 * If there is a file and the buffer is not modified, we reopen the file.      *
 *******************************************************************************/
  gint x;
  katoob_debug (__FUNCTION__);
  if (!katoob_document_get_file (doc))
    {
      katoob_debug ("No file for document.");
      if (!katoob_document_get_modified (doc))	/* A blank document ;) */
	{
	  katoob_debug ("Blank document.");
	  coverted_ok = 0;
	  return;
	}
      else
	{
	  x =
	    katoob_create_question (_
				    ("You have modified the current text, Katoob'll convert the current text to your default encoding and then convert it back to your choosen encoding. Continue ?"));
	  switch (x)
	    {
	    case GTK_RESPONSE_YES:
	      {
		gchar *new_text = NULL;
		gchar *text = katoob_document_get_text (doc);

		if (!text)
		  {
		    g_warning ("This shouldn't happen!");
		    coverted_ok = 1;
		    return;
		  }
		new_text = katoob_changed_text_encoding (text, enc);
		g_free (text);
		if (!new_text)	/* err already displayed! */
		  {
		    coverted_ok = 1;
		    return;
		  }
		katoob_document_set_text (doc, new_text);
		katoob_document_reset_undo_redo (doc);
/*		katoob_document_set_modified (doc, FALSE); */
		g_free (new_text);
		coverted_ok = 0;
		return;
	      }
	    case GTK_RESPONSE_NO:
	      {
		coverted_ok = 1;
		return;
	      }
	    }
	}
    }
  if (katoob_document_get_modified (doc))
    {
      katoob_debug ("Modified document.");
/* Prompt the user. */
      x =
	katoob_create_question (_
				("You are changing the encoding of a modified file, This'll also clear the undo & redo history, Should katoob reopen the file?"));
      switch (x)
	{
	case GTK_RESPONSE_YES:
	  {
/***************************************************************************
 * Get the file contents, Stop the undo/redo, Clear the buffer, Insert the * 
 * contents, enable the undo/redo                                          *
 ***************************************************************************/
	    gchar *content = NULL;
	    gchar *content2 = NULL;

	    if (!katoob_create_file_if_required
		(katoob_document_get_file (doc)))
	      {
		coverted_ok = 1;
		return;
	      }
	    if (!katoob_file_get_contents
		(katoob_document_get_file (doc), &content))
	      {
		coverted_ok = 1;
		return;
	      }
	    if (content)
	      {
		content2 = katoob_text_to_utf8 (content, doc);
		g_free (content);
		if (!content2)
		  {
		    coverted_ok = 1;
		    return;
		  }
	      }
	    katoob_document_set_text (doc, content2);
	    if (content2)
	      {
		g_free (content2);
	      }
	    katoob_document_reset_undo_redo (doc);
	    katoob_document_set_modified (doc, FALSE);
	    coverted_ok = 0;
	    return;
	  }
	case GTK_RESPONSE_NO:
	  {
	    coverted_ok = 1;
	    return;
	  }
	}
    }
  else
    {
      gchar *content = NULL;
      gchar *content2 = NULL;

      katoob_debug ("Not a modified document.");
      if (!katoob_create_file_if_required (katoob_document_get_file (doc)))
	{
	  coverted_ok = 1;
	  return;
	}
      if (!katoob_file_get_contents
	  (katoob_document_get_file (doc), &content))
	{
	  coverted_ok = 1;
	  return;
	}

/* If the file is empty, content is set to NULL */

      if (!content)
	{
	  coverted_ok = 1;
	  return;
	}
      else
	{
	  content2 = katoob_text_to_utf8 (content, doc);
	  g_free (content);
	  if (!content2)
	    {
	      coverted_ok = 1;
	      return;
	    }
	}
      katoob_document_set_text (doc, content2);
      if (content2)
	{
	  g_free (content2);
	}
      katoob_document_reset_undo_redo (doc);
      katoob_document_set_modified (doc, FALSE);
      coverted_ok = 0;
    }
}
